function W = GMMweightsNW(Params, Model)
% =============================================================================================
% Optimal weighting matrix for CKLS nested models
% 
% INPUT: Params, vector, vector of estimated parameters
%        Model, structure, see RunAssignment2   
% OUTPUT: W, matrix, optimal weighting matrix
%
% Technical Computing Prague 2008
% Kamil Kladivko 
% email: kladivko@gmail.com
% =============================================================================================

Data = Model.Data;
TimeStep = Model.TimeStep;
q = Model.q;
a = Params(1);
b = Params(2);
DataF = Data(2:end);
DataL = Data(1:end-1);
Gamma = zeros(4,4,q+1);

% Construct sample moment functions
switch Model.Name
   case 'CKLS'
        sigma = Params(3);
        gamma = Params(4);
        g1t = DataF - a - b*DataL;
        g2t = (DataF - a - b*DataL).^2 - sigma^(2*gamma)*DataL.*TimeStep;
        g3t = (DataF - a -b*DataL).*DataL;
        g4t = ((DataF - a - b*DataL).^2 - sigma^(2*gamma)*DataL.*TimeStep).*DataL; 
    case 'CIR'
        sigma = Params(3);
        g1t = DataF - a - b*DataL;
        g2t = (DataF - a - b*DataL).^2 - sigma^2*DataL.*TimeStep;
        g3t = (DataF - a -b*DataL).*DataL;
        g4t = ((DataF - a - b*DataL).^2 - sigma^2*DataL.*TimeStep).*DataL;              
    case 'Vasicek'
        sigma = Params(3);
        g1t = DataF - a - b*DataL;
        g2t = (DataF - a - b*DataL).^2 - sigma^2*TimeStep;
        g3t = (DataF - a -b*DataL).*DataL;
        g4t = ((DataF - a - b*DataL).^2 - sigma^2*TimeStep).*DataL;       
end
gt = [g1t g2t g3t g4t];
Nobs = length(g1t);

% Calculate the Newey-West estimate of the spectral density matrix with q lags
gt = gt - repmat(mean(gt), Nobs, 1);  
for v = 0 : q   
    gtF = gt(1+v:end, :); 
    gtL = gt(1:end-v, :);
    Gamma(:,:,v+1) = (gtF'*gtL)./Nobs;           
end
S = Gamma(:,:,1);
for v = 1 : q
    Snext = (1-v/(q+1))*(Gamma(:,:,v+1) + Gamma(:,:,v+1)');
    S = S + Snext;        
end
W = inv(S);
end