function d = MomentsJacobian(Params, Model)
% =============================================================================================
% Jacobian matrix for testing parameters significance (t-test)
% 
% INPUT: Params, vector, vector of estimated parameters
%        Model, structure, see RunAssignment2   
% OUTPUT: d, Jacobian matrix
%
% Technical Computing Prague 2008
% Kamil Kladivko 
% email: kladivko@gmail.com
% =============================================================================================

TimeStep = Model.TimeStep;
        
Data = Model.Data;
DataF = Data(2:end);
DataL = Data(1:end-1);
Nobs = length(DataL);

switch Model.Name
    case 'CKLS' 
        a = Params(1);
        b = Params(2);
        sigma = Params(3);
        gamma = Params(4);
        
        g1a = -Nobs;
        g2a = -2*sum(DataF - a - b*DataL);
        g3a = -sum(DataL);
        g4a = -2*sum((DataF - a - b*DataL).*DataL);

        g1b = -sum(DataL);
        g2b = -2*sum((DataF - a - b*DataL).*DataL);
        g3b = -sum(DataL.^2);
        g4b = -2*sum((DataF - a - b*DataL).*DataL.^2);

        g1s = 0;
        g2s = -TimeStep*sum(DataL.^(2*gamma));
        g3s = 0;
        g4s = -TimeStep*sum(DataL.^(2*gamma+1));

        g1g = 0;
        g2g = -2*sigma^2*TimeStep*sum(log(DataL).*DataL.^(2*gamma));
        g3g = 0;
        g4g = -2*sigma^2*TimeStep*sum(log(DataL).*DataL.^(2*gamma+1));
        
        d = [g1a g1b g1s g1g;...
             g2a g2b g2s g2g;...
             g3a g3b g3s g3g;...
             g4a g4b g4s g4g];
        d = d./Nobs;
    case 'CIR'
        a = Params(1);
        b = Params(2);
        
        g1a = -Nobs;
        g2a = -2*sum(DataF - a - b*DataL);
        g3a = -sum(DataL);
        g4a = -2*sum((DataF - a - b*DataL).*DataL);

        g1b = -sum(DataL);
        g2b = -2*sum((DataF - a - b*DataL).*DataL);
        g3b = -sum(DataL.^2);
        g4b = -2*sum((DataF - a - b*DataL).*DataL.^2);

        g1s = 0;
        g2s = -sum(TimeStep*DataL);
        g3s = 0;
        g4s = -sum(TimeStep*DataL.*DataL);

        d = [g1a g1b g1s;...
             g2a g2b g2s;...
             g3a g3b g3s;...
             g4a g4b g4s];         
        d = d./Nobs;
    case 'Vasicek'
        a = Params(1);
        b = Params(2);

        g1a = -Nobs;
        g2a = -2*sum(DataF - a - b*DataL);
        g3a = -sum(DataL);
        g4a = -2*sum((DataF - a - b*DataL).*DataL);

        g1b = -sum(DataL);
        g2b = -2*sum((DataF - a - b*DataL).*DataL);
        g3b = -sum(DataL.^2);
        g4b = -2*sum((DataF - a - b*DataL).*DataL.^2);

        g1s = 0;
        g2s = -TimeStep*Nobs;
        g3s = 0;
        g4s = -sum(TimeStep*DataL);

        d = [g1a g1b g1s;...
             g2a g2b g2s;...
             g3a g3b g3s;...
             g4a g4b g4s];         
        d = d./Nobs;        
end
end