function R2 = R2measures(Model, Results)
% =============================================================================================
% R2 measures
% 
% INPUT: Model, structure, see RunAssignment2  
%        Results, structure, see GMMestimation
% OUTPUT: R2, structure
%         R2.Rsquare1, forecast power for interest rate changes, see CKLS p. 1219
%         R2.Rsquare2, forecast power for squared interest rate changes, see CKLS p. 1219
%
% Technical Computing Prague 2008
% Kamil Kladivko 
% email: kladivko@gmail.com
% ============================================================================================

Data = Model.Data;
TimeStep = Model.TimeStep;
alpha = Results.Params(1);
beta = Results.Params(2);
sigma2 = Results.Params(3);
gamma = Results.Params(4);

CondEdata = alpha*TimeStep + (1+beta*TimeStep).*Data;
CondEddata = CondEdata - Data;
Realddata = diff(Data);

CondVARdata = sigma2*Data.^(2*gamma)*TimeStep;
RealVARdata = Realddata.^2;

% "forecast power" - calcluate R2 of the OLS of conditional values implied by the selected
% model on observed values
y = Realddata;
x = CondEddata(1:end-1);
b1 = x\y;
Rsquare1 = sum((b1.*x-mean(y)).^2)/sum((y-mean(y)).^2);

y = RealVARdata;
x = CondVARdata(1:end-1);
b1 = x\y;
Rsquare2 = sum((b1.*x-mean(y)).^2)/sum((y-mean(y)).^2);

R2.Rsquare1 = Rsquare1;
R2.Rsquare2 = Rsquare2;

if strcmp(Model.Disp, 'y')
    fprintf(' Rsquare 1 = %+2.4f\n', Rsquare1);
    fprintf(' Rsquare 2 = %+2.4f\n', Rsquare2);
    figure
    plot(1:length(Realddata), abs(Realddata), 'b', 1:length(Realddata), sqrt(CondVARdata(1:end-1)), 'r')
    title('Interest Rate Volatility, page 1221')
    xlim([1,length(Realddata)])
    legend('Actual', 'Forecast')
end

end