% =============================================================================================
%                            RUN GMM ESTIMATION ROUTINE OF THE CKLS MODEL
% Chan, K.C., Karolyi, Andrew G., Longstaff, Francis A. and Sanders, Anthony B., 1992, An
% Empirical Comparison of Alternative Models of the Short-Term Interest Rate, The Journal
% of Finance 47, 12091227.
%
% Technical Computing Prague 2008
% Kamil Kladivko 
% email: kladivko@gmail.com
%
% =============================================================================================

clc
clear variables
close all

% INPUT
fid=fopen('ckls.dat'); 
data = textscan(fid, '%d %*d %f', -1, 'headerlines', 4); 
fclose(fid); 

Model.Data = data{2}./100;  % CRSP data
Model.Name = 'CIR';        % 'CKLS' (unrestricted)|'CIR'|'Vasicek' 
Model.TimeStep = 1/12;      % recommended: 1/12 for monthly data, 1/250 for daily data, etc
Model.MatlabDisp = 'off';   % 'off'|'iter'|'notify'|'final'  (default: off)
Model.Disp = 'y';           % 'y'|'n' (Print results in Matlab's command window, draws graphs)
Model.Iters = 1;            % # of iterations of the weighting matrix (traditionally = 1) 
Model.q = 12;               % # of lags in the spectral density matrix estimation,
                            % Model.q = 0 reduces the spectral density matrix to the sample
                            % covariance matrix               

% ESTIMATION
Results = GMMestimation(Model);

% R2 measures
R2 = R2measures(Model, Results); % calculate the R2 measures
